"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
exports.__esModule = true;
exports.LinkedList = exports.ListNode = void 0;
var types_1 = require("./types");
var iterators_1 = require("../iterators");
/**
 * A node of the double-ended linked list.
 */
var ListNode = /** @class */ (function () {
    function ListNode(value, root, prev, next) {
        if (prev === void 0) { prev = null; }
        if (next === void 0) { next = null; }
        this.value = value;
        this.root = root;
        this.next = next;
        this.prev = prev;
    }
    /**
     * Inserts supplied value before the current node.
     * @param value
     *
     */
    ListNode.prototype.insertPrev = function (value) {
        var node = new ListNode(value, this.root, this.prev, this);
        if (this.prev !== null)
            this.prev.next = node;
        return (this.prev = node);
    };
    /**
     * Inserts supplied value after the current node.
     * @param value
     *
     */
    ListNode.prototype.insertNext = function (value) {
        var node = new ListNode(value, this.root, this, this.next);
        if (this.next !== null)
            this.next.prev = node;
        return (this.next = node);
    };
    /**
     * Disconnects current node from its predecessor and successor.
     *
     */
    ListNode.prototype.unlink = function () {
        if (this.next !== null)
            this.next.prev = this.prev;
        if (this.prev !== null)
            this.prev.next = this.next;
        this.prev = this.next = null;
    };
    /**
     * Disconnects current node from its predecessor, successor and root.
     *
     */
    ListNode.prototype.destroy = function () {
        this.unlink();
        this.root = null;
    };
    return ListNode;
}());
exports.ListNode = ListNode;
/**
 * Double-ended linked list.
 */
var LinkedList = /** @class */ (function (_super) {
    __extends(LinkedList, _super);
    function LinkedList(values) {
        var e_1, _a;
        if (values === void 0) { values = iterators_1.EMPTY_ITER; }
        var _this = _super.call(this) || this;
        _this.length = 0;
        _this.head = null;
        _this.tail = null;
        try {
            for (var values_1 = __values(values), values_1_1 = values_1.next(); !values_1_1.done; values_1_1 = values_1.next()) {
                var value = values_1_1.value;
                _this.pushBack(value);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (values_1_1 && !values_1_1.done && (_a = values_1["return"])) _a.call(values_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return _this;
    }
    /**
     * Pushes an node to the back of the list.
     * @param node
     *
     */
    LinkedList.prototype.pushBack = function (node) {
        this.length += 1;
        if (this.tail !== null) {
            this.tail = this.tail.insertNext(node);
        }
        else {
            this.head = this.tail = new ListNode(node, this);
        }
        return this.tail;
    };
    /**
     * Pushes an node to the front of the list.
     * @param node
     *
     */
    LinkedList.prototype.pushFront = function (node) {
        this.length += 1;
        if (this.head !== null) {
            this.head = this.head.insertPrev(node);
        }
        else {
            this.head = this.tail = new ListNode(node, this);
        }
        return this.head;
    };
    /**
     * Moves node to the front of the queue.
     * Returns `false` if node doesn't belong to the given list.
     * @param node
     *
     */
    LinkedList.prototype.moveFront = function (node) {
        if (!this.contains(node)) {
            return false;
        }
        else if (node === this.head) {
            return true;
        }
        else if (node === this.tail) {
            this.tail = this.tail.prev;
        }
        node.unlink();
        node.next = this.head;
        this.head.prev = node;
        this.head = node;
        return true;
    };
    /**
     * Moves node to the back of the queue.
     * Returns `false` if node doesn't belong to the given list.
     * @param node
     *
     */
    LinkedList.prototype.moveBack = function (node) {
        if (!this.contains(node)) {
            return false;
        }
        else if (node === this.tail) {
            return true;
        }
        else if (node === this.head) {
            this.head = this.head.next;
        }
        node.unlink();
        node.prev = this.tail;
        this.tail.next = node;
        this.tail = node;
        return true;
    };
    /**
     * Peeks a value from the front of the queue.
     * Returns either item or `null` if queue is empty.
     *
     */
    LinkedList.prototype.peekFront = function () {
        return this.head && this.head.value;
    };
    /**
     * Peeks a value from the back of the queue.
     * Returns either item or `null` if queue is empty.
     *
     */
    LinkedList.prototype.peekBack = function () {
        return this.tail && this.tail.value;
    };
    /**
     * Takes a value from the back of the queue.
     * Returns either item or `null` if queue is empty.
     *
     */
    LinkedList.prototype.takeBack = function () {
        if (this.tail === null)
            return null;
        this.length--;
        var node = this.tail;
        if (node === this.head) {
            this.head = null;
        }
        if (node !== null) {
            this.tail = this.tail.prev;
            node.destroy();
            return node.value;
        }
        return null;
    };
    /**
     * Takes front node from the list.
     * Returns `null` if list has no nodes.
     *
     */
    LinkedList.prototype.takeFront = function () {
        if (this.head === null)
            return null;
        this.length--;
        var node = this.head;
        if (node === this.tail) {
            this.tail = null;
        }
        if (node !== null) {
            this.head = this.head.next;
            node.destroy();
            return node.value;
        }
        return null;
    };
    /**
     * Inserts given value after the supplied raw linked list node.
     * Returns `null` in case supplied node doesn't belong to this list.
     * @param node
     * @param value
     *
     */
    LinkedList.prototype.insertAfter = function (node, value) {
        if (!this.contains(node)) {
            return null;
        }
        this.length++;
        var inserted = node.insertNext(value);
        if (node === this.tail) {
            this.tail = inserted;
        }
        return inserted;
    };
    /**
     * Inserts given value before the supplied raw linked list node.
     * Returns `null` in case supplied node doesn't belong to this list.
     * @param node
     * @param value
     *
     */
    LinkedList.prototype.insertBefore = function (node, value) {
        if (!this.contains(node)) {
            return null;
        }
        this.length++;
        var inserted = node.insertPrev(value);
        if (node === this.head) {
            this.head = inserted;
        }
        return inserted;
    };
    /**
     * Returns `true` if supplied node belongs to the list.
     * @param node
     */
    LinkedList.prototype.contains = function (node) {
        return node.root === this;
    };
    /**
     * Removes given node from the list.
     * Returns `false` if node doesn't belong to the given list.
     * @param node
     */
    LinkedList.prototype.remove = function (node) {
        if (!this.contains(node)) {
            return false;
        }
        this.length--;
        if (node === this.head && this.head !== null) {
            this.head = this.head.next;
        }
        if (node === this.tail && this.tail !== null) {
            this.tail = this.tail.prev;
        }
        node.destroy();
        return true;
    };
    /**
     * Returns amount of items stored in the list.
     *
     */
    LinkedList.prototype.len = function () {
        return this.length;
    };
    /**
     * Returns an iterator over collection values starting from the end.
     */
    LinkedList.prototype.valuesBack = function () {
        var _a;
        var that = this;
        return _a = {},
            _a[Symbol.iterator] = function () {
                var tail = that.tail;
                return {
                    next: function () {
                        if (tail !== null) {
                            var value = tail.value;
                            tail = tail.prev;
                            return { value: value, done: false };
                        }
                        else {
                            return iterators_1.ITER_DONE_VALUE;
                        }
                    }
                };
            },
            _a;
    };
    /**
     * Returns an iterator over collection values starting from the beginning.
     */
    LinkedList.prototype.valuesFront = function () {
        var _a;
        var that = this;
        return _a = {},
            _a[Symbol.iterator] = function () {
                var head = that.head;
                return {
                    next: function () {
                        if (head !== null) {
                            var value = head.value;
                            head = head.next;
                            return { value: value, done: false };
                        }
                        else {
                            return iterators_1.ITER_DONE_VALUE;
                        }
                    }
                };
            },
            _a;
    };
    return LinkedList;
}(types_1.OrderedCollection));
exports.LinkedList = LinkedList;
//# sourceMappingURL=LinkedList.js.map