"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
exports.__esModule = true;
var mimic_fn_1 = require("mimic-fn");
var StorageContext_1 = require("./StorageContext");
var Root_1 = require("./Root");
var value_1 = require("../value");
var utils_1 = require("../utils");
var slice = Array.prototype.slice;
/**
 * Memoizes provided function returning wrapped version of it.
 * Result function will return value without calling the supplied function if it's present in the cache for the supplied arguments according to `Same-value-zero` algorithm.
 * If no value is found, the underlying function will be called with provided arguments.
 * @param func
 * @param params
 */
function memoize(func, _a) {
    if (_a === void 0) { _a = utils_1.EMPTY_OBJECT; }
    var _b = _a.length, length = _b === void 0 ? func.length : _b, _c = _a.checkLast, checkLast = _c === void 0 ? true : _c, params = __rest(_a, ["length", "checkLast"]);
    var recomputate = function () {
        ++resultFunction.recomputations;
        return func.apply(func, arguments);
    };
    var resultFunction;
    if (length === 0) {
        var value_2 = value_1.NO_VALUE;
        resultFunction = function cachedFunction() {
            if (value_2 !== value_1.NO_VALUE) {
                return value_2;
            }
            else {
                return (value_2 = recomputate.apply(this, arguments));
            }
        };
    }
    else {
        var ctx = new StorageContext_1.StorageContext(params);
        var root_1 = new Root_1.Root({ length: length, checkLast: checkLast }, ctx);
        resultFunction = function cachedFunction() {
            var argsLength = arguments.length;
            var output;
            if (argsLength === length) {
                output = root_1.getOrInsertWith(arguments, recomputate);
                // If we received a greater amount of arguments, slice it
            }
            else if (argsLength > length) {
                output = root_1.getOrInsertWith(slice.call(arguments, 0, length), recomputate);
                // Otherwise don't use cache
            }
            else {
                output = recomputate.apply(func, arguments);
            }
            return output;
        };
    }
    try {
        void (0, mimic_fn_1["default"])(resultFunction, func);
    }
    finally {
        resultFunction.recomputations = 0;
        return resultFunction;
    }
}
exports["default"] = memoize;
//# sourceMappingURL=index.js.map