"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
exports.__esModule = true;
exports.ObjectStorage = exports.Key = void 0;
var value_1 = require("../value");
var value_2 = require("../value");
var Storage_1 = require("../base/Storage");
var iterators_1 = require("../iterators");
/**
 * A key for the `ObjectStorage`.
 */
var Key = /** @class */ (function () {
    function Key(value) {
        if (!(0, value_1.isPrimitiveValue)(value)) {
            throw new TypeError("Invalid value: ".concat(value, ", expected primitive"));
        }
        var type = Key.valueType(value);
        this.type = type;
        this.value = value;
    }
    /**
     * Returns type index for the provided value.
     */
    Key.valueType = function (value) {
        var type = typeof value;
        switch (type) {
            case "number":
                return 0;
            case "bigint":
                return 1;
            case "string":
                return 2;
            case "undefined":
                return 3;
            case "symbol":
                return 4;
            case "boolean":
                return 5;
            default:
                if (value === null)
                    return 6;
        }
        throw new TypeError("Invalid value ".concat(value, " with type ").concat(type, ", expected primitive"));
    };
    /**
     * Creates new value using the provided type index and stringified representation.
     */
    Key.valueFromStringified = function (type, value) {
        switch (type) {
            case 0:
                return new Key(Number(value));
            case 1:
                return new Key(BigInt(value));
            case 2:
                return new Key(value);
            case 3:
                return new Key(undefined);
            case 4:
                return new Key(Symbol(value));
            case 5:
                return new Key(value === "true");
            case 6:
                return new Key(null);
        }
        throw new TypeError("Invalid type ".concat(type, " of value ").concat(value));
    };
    /**
     * Converts underlying value to the string representation.
     */
    Key.prototype.toString = function () {
        return "".concat(this.type).concat(this.value);
    };
    /**
     * Instantiates value from string.
     */
    Key.fromString = function (stringified) {
        var type = +stringified[0];
        return Key.valueFromStringified(type, stringified.slice(1));
    };
    return Key;
}());
exports.Key = Key;
/**
 * Key-value storage for values with primitive keys based on an `Object`.
 */
var ObjectStorage = /** @class */ (function (_super) {
    __extends(ObjectStorage, _super);
    function ObjectStorage(params, rootPath) {
        var _this = _super.call(this, params, rootPath) || this;
        _this.length = 0;
        _this.map = Object.create(null);
        return _this;
    }
    /**
     * Returns amount of items stored in a map.
     *
     */
    ObjectStorage.prototype.len = function () {
        return this.length;
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param rawKey
     *
     */
    ObjectStorage.prototype.has = function (rawKey) {
        var keyStr = new Key(rawKey).toString();
        return keyStr in this.map;
    };
    /**
     * Retrieves an item corresponding to the supplied key.
     * @param rawKey
     *
     */
    ObjectStorage.prototype.get = function (rawKey) {
        var keyStr = new Key(rawKey).toString();
        if (keyStr in this.map) {
            return this.map[keyStr];
        }
        else {
            return value_2.NO_VALUE;
        }
    };
    /**
     * Drops an item corresponding to the supplied key.
     * @param key
     *
     */
    ObjectStorage.prototype.drop = function (rawKey) {
        var keyStr = new Key(rawKey).toString();
        var value = value_2.NO_VALUE;
        if (keyStr in this.map) {
            this.length--;
            value = this.map[keyStr];
            delete this.map[keyStr];
        }
        return value;
    };
    /**
     * Associates supplied item with the key.
     * @param rawKey
     * @param value
     *
     */
    ObjectStorage.prototype.set = function (rawKey, value) {
        var keyStr = new Key(rawKey).toString();
        var has = keyStr in this.map;
        if (!has) {
            this.length++;
            this.map[keyStr.toString()] = value;
        }
    };
    /**
     * Removes all items from the storage.
     *
     */
    ObjectStorage.prototype.clear = function () {
        this.length = 0;
        this.map = Object.create(null);
    };
    /**
     * Returns an iterator over the entries.
     *
     */
    ObjectStorage.prototype.entries = function () {
        return (0, iterators_1.map)(function (_a) {
            var _b = __read(_a, 2), key = _b[0], value = _b[1];
            return ({
                key: Key.fromString(key).value,
                value: value
            });
        }, Object.entries(this.map));
    };
    return ObjectStorage;
}(Storage_1.Storage));
exports.ObjectStorage = ObjectStorage;
//# sourceMappingURL=ObjectStorage.js.map