"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.FIFO = void 0;
var CacheStrategy_1 = require("../base/CacheStrategy");
var collections_1 = require("../collections");
var value_1 = require("../value");
var utils_1 = require("../utils");
var iterators_1 = require("../iterators");
/**
 * `FIFO` - `F`irst `I`n - `F`irst `O`ut cache replacement policy.
 */
var FIFO = /** @class */ (function (_super) {
    __extends(FIFO, _super);
    function FIFO(capacity) {
        var _this = _super.call(this, capacity) || this;
        _this.queue = new collections_1.SingleKeyQueue();
        return _this;
    }
    /**
     * Returns amount of stored items.
     *
     */
    FIFO.prototype.len = function () {
        return this.queue.len();
    };
    /**
     * Returns `true` if given item exists in the queue.
     * @param node
     *
     */
    FIFO.prototype.has = function (node) {
        return this.queue.has(node);
    };
    /**
     * Removes supplied item from the queue.
     * @param node
     *
     */
    FIFO.prototype.drop = function (node) {
        var has = this.queue.has(node);
        if (has) {
            this.queue.drop(node);
        }
        return has;
    };
    /**
     * Records write access of the supplied item.
     * @param value
     *
     */
    FIFO.prototype.write = function (value) {
        var res = _super.prototype.write.call(this, value);
        var item = this.queue.get(value);
        if (item !== value_1.NO_VALUE) {
            var moved = this.queue.moveBack(item);
            if (!moved) {
                throw new Error("`FIFO`: failed to move the cache entry");
            }
            return res;
        }
        else {
            var pushed = this.queue.pushBack(value);
            if (pushed === value_1.NO_VALUE) {
                throw new Error("`FIFO`: failed to push a new cache entry");
            }
            return res.chain(utils_1.Result.added((0, iterators_1.once)(value)));
        }
    };
    /**
     * Peeks a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    FIFO.prototype.peek = function () {
        return this.queue.peekKeyFront();
    };
    /**
     * Removes an item from the beginning of the queue.
     *
     */
    FIFO.prototype.take = function () {
        return this.queue.takeKeyFront();
    };
    return FIFO;
}(CacheStrategy_1.CacheStrategy));
exports.FIFO = FIFO;
//# sourceMappingURL=fifo.js.map