"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.LRU = void 0;
var CacheStrategy_1 = require("../base/CacheStrategy");
var collections_1 = require("../collections");
var value_1 = require("../value");
var utils_1 = require("../utils");
var iterators_1 = require("../iterators");
/**
 * `L`east `R`ecently `U`sed cache replacement policy.
 */
var LRU = /** @class */ (function (_super) {
    __extends(LRU, _super);
    function LRU(capacity) {
        var _this = _super.call(this, capacity) || this;
        _this.queue = new collections_1.SingleKeyQueue();
        return _this;
    }
    /**
     * Returns amount of keys (references) stored in a map.
     *
     */
    LRU.prototype.len = function () {
        return this.queue.len();
    };
    /**
     * Returns `true` if given item exists in the queue.
     * @param node
     *
     */
    LRU.prototype.has = function (node) {
        return this.queue.has(node);
    };
    /**
     * Removes supplied item from the queue.
     * @param node
     *
     */
    LRU.prototype.drop = function (node) {
        return this.queue.drop(node) !== value_1.NO_VALUE;
    };
    /**
     * Records read access of the supplied item.
     * @param value
     *
     */
    LRU.prototype.read = function (value) {
        return _super.prototype.read.call(this, value).chain(this.touch(value));
    };
    /**
     * Records write access of the supplied item.
     * @param value
     *
     */
    LRU.prototype.write = function (value) {
        return _super.prototype.write.call(this, value).chain(this.touch(value));
    };
    LRU.prototype.touch = function (value) {
        var item = this.queue.get(value);
        if (item !== value_1.NO_VALUE) {
            var moved = this.queue.moveBack(item);
            if (!moved) {
                throw new Error("`LRU`: failed to move the cache entry");
            }
            return utils_1.Result.empty();
        }
        else {
            var pushed = this.queue.pushBack(value);
            if (pushed === value_1.NO_VALUE) {
                throw new Error("`LRU`: failed to push a new cache entry");
            }
            return utils_1.Result.added((0, iterators_1.once)(value));
        }
    };
    /**
     * Peeks a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    LRU.prototype.peek = function () {
        return this.queue.peekKeyFront();
    };
    /**
     * Removes an item from the beginning of the queue.
     *
     */
    LRU.prototype.take = function () {
        return this.queue.takeKeyFront();
    };
    return LRU;
}(CacheStrategy_1.CacheStrategy));
exports.LRU = LRU;
//# sourceMappingURL=lru.js.map