"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
exports.__esModule = true;
exports.Result = exports.ParentPath = exports.HasCapacity = exports.HasLength = exports.EMPTY_OBJECT = exports.EMPTY_ARRAY = exports.mapImplemented = exports.noop = void 0;
var iterators_1 = require("./iterators");
var value_1 = require("./value");
/**
 * Does nothing.
 */
var noop = function () { };
exports.noop = noop;
/**
 * `true` if `Map` is implemented.
 */
var mapImplemented = function () {
    return Boolean(Map && Map.prototype && typeof Map.prototype.entries === "function");
};
exports.mapImplemented = mapImplemented;
/**
 * An empty array. MUST NOT BE MODIFIED.
 */
exports.EMPTY_ARRAY = [];
/**
 * An empty object. MUST NOT BE MODIFIED.
 */
exports.EMPTY_OBJECT = {};
/**
 * Describes a container having a length.
 */
var HasLength = /** @class */ (function () {
    function HasLength() {
    }
    /**
     * Returns `true` if the underlying container is empty (has zero length).
     */
    HasLength.prototype.isEmpty = function () {
        return this.len() === 0;
    };
    return HasLength;
}());
exports.HasLength = HasLength;
/**
 * Describes a container having both capacity and length.
 */
var HasCapacity = /** @class */ (function (_super) {
    __extends(HasCapacity, _super);
    /**
     * Creates a new container with supplied capacity.
     * In case capacity is equal to zero, an error will be thrown.
     * @param capacity
     */
    function HasCapacity(capacity) {
        var _this = _super.call(this) || this;
        if (capacity === 0) {
            throw new Error("Capacity can't be equal to zero");
        }
        _this._capacity = capacity;
        return _this;
    }
    /**
     * Returns capacity of the underlying container.
     *
     *
     */
    HasCapacity.prototype.capacity = function () {
        return this._capacity;
    };
    /**
     * Returns `true` if the length of the underlying container reached its capacity.
     *
     *
     */
    HasCapacity.prototype.isFull = function () {
        return this.capacity() < this.len();
    };
    /**
     * Returns `true` if the length of the underlying container will reach (or already reached) its capacity.
     *
     *
     */
    HasCapacity.prototype.willBeFull = function () {
        return this.capacity() < this.len() + 1;
    };
    return HasCapacity;
}(HasLength));
exports.HasCapacity = HasCapacity;
/**
 * The path from the parent to the child.
 * If the key is a `NO_VALUE`, then the child is stored under the key equal to itself.
 */
var ParentPath = /** @class */ (function () {
    function ParentPath(parent, key) {
        this.parent = parent;
        this.key = key;
    }
    /// Enforces parent to drop the supplied child. Its value will be used only in case if key is absent.
    ParentPath.prototype.drop = function (child) {
        this.parent.drop(this.key === value_1.NO_VALUE ? child : this.key);
    };
    return ParentPath;
}());
exports.ParentPath = ParentPath;
/**
 * Contains removed/added entities.
 */
var Result = /** @class */ (function () {
    function Result(removed, added) {
        if (removed === void 0) { removed = iterators_1.EMPTY_ITER; }
        if (added === void 0) { added = iterators_1.EMPTY_ITER; }
        this.removed = removed;
        this.added = added;
    }
    /** Creates a result containing added entities */
    Result.added = function (added) {
        return new Result(void 0, added);
    };
    /** Creates a result containing removed entities */
    Result.removed = function (removed) {
        return new Result(removed);
    };
    /**
     * Creates a result containing removed and added entities.
     * @param removed
     * @param added
     */
    Result.removedAdded = function (removed, added) {
        return new Result(removed, added);
    };
    /**
     * Creates an empty result.
     */
    Result.empty = function () {
        return Result.EMPTY_RESULT;
    };
    /**
     * Appends remove/added items from the supplied result to the current result.
     * @param result
     */
    Result.prototype.chain = function (result) {
        if (result === Result.EMPTY_RESULT) {
            return this;
        }
        else if (this === Result.EMPTY_RESULT) {
            return result;
        }
        return new Result((0, iterators_1.chain)(this.removed, result.removed), (0, iterators_1.chain)(this.added, result.added));
    };
    /**
     * Executes given function for each added item.
     * @param fn
     *
     */
    Result.prototype.forEachAdded = function (fn) {
        var e_1, _a;
        try {
            for (var _b = __values(this.added), _c = _b.next(); !_c.done; _c = _b.next()) {
                var added = _c.value;
                fn(added);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return this;
    };
    /**
     * Executes given function for each removed item.
     * @param fn
     *
     */
    Result.prototype.forEachRemoved = function (fn) {
        var e_2, _a;
        try {
            for (var _b = __values(this.removed), _c = _b.next(); !_c.done; _c = _b.next()) {
                var removed = _c.value;
                fn(removed);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return this;
    };
    /**
     * Maps given result over removed and added items.
     * @param fn
     */
    Result.prototype.map = function (fn) {
        return new Result((0, iterators_1.map)(fn, this.removed), (0, iterators_1.map)(fn, this.added));
    };
    Result.EMPTY_RESULT = new Result();
    return Result;
}());
exports.Result = Result;
//# sourceMappingURL=utils.js.map